import os, sys
import json

class Config:

    case_sensitive = True # 大小写敏感
    
    word_coef = 1 # 词预测系数
    order_coef = 0.1 # 词排序系数
    ranking_coef = 0.1 # 句子排序系数
    
    use_glove = False # 是否采用预训练词向量
    
    model_dir = './model/scratch.w2v.200k/GRU.case/word-1-order-0.1.maxpooling/'
    if not os.path.isdir(model_dir):
        os.mkdir(model_dir)
    tensorboard_tdir = model_dir + 'train/'
    if not os.path.isdir(tensorboard_tdir):
        os.mkdir(tensorboard_tdir)
    tensorboard_vdir = model_dir + 'valid/'
    if not os.path.isdir(tensorboard_vdir):
        os.mkdir(tensorboard_vdir)

    '''rnn parameters'''
    hidden_size = 1200
    rnn_layers = 1
    bidirectional = True
    
    '''cnn parameters'''
    kernel_size = 3
    channels = 1600
    cnn_layers = 3
    dilations = [1, 1, 1] # 每层的膨胀系数
    
    '''transformer parameters'''
    n_layers = 6
    n_head = 10
    transformer_hidden_size = 720
    d_inner=2048
    d_k=72
    d_v=72
    
    '''crnn parameters'''
    crnn_kernel_sizes = [3, 4, 5]
    crnn_channels = [100, 100, 100]
    
    
    mapping_dim = 512
    dropout = 0.3
    pooling_type = 'maxpooling'
    enc = 'GRU'
    activation = 'ReLU'
    batch_size = 400
    lr = 3e-4
    lr_decay_batch = 20000 # 训练若干个batch后验证集没有变好，就降低lr
    weight_decay = 0 # L2
    epoch = 1000
    
    normfunc = 'not xariver'
    
    valid_per_batch = 1000
    print_per_batch = 25
    save_per_second = 3600 * 5
    early_stopping_batch = 10000
    
    eos_id = 1
    bos_id = 0
    unk_id = 2
    
    voc_size = 50000 * 4
    ranking_margin = 0.25 # 句子排序margin_loss margin
    ns_para_ranking = 0 # 句子排序负例
    text_window = 3 # 上下文窗口范围
    min_num_ranking_seq = 2 # 句子排序最少数目
    std_word_neg_num = 15 # word预测负采样个数
    std_word_pos_num = 3 # word预测正例个数
    std_order_neg_num = 1 # 暂时没用，语序预测负例是将正例进行打乱，每个正例对应一个负例，因为需要重新enc，资源消耗大
    std_order_pos_num = 1 # 暂时没用，语序预测正例和词汇预测共用一个sentvec
    std_ranking_neg_num = 2 # 句子排序负例个数
    max_sent_len = 30
    min_sent_len = 5
    
    train_path = '../data/bookcorpus/train.token.lmdb'
    valid_path = '../data/bookcorpus/valid.token.lmdb'
        
    
    if use_glove:
        embedding_dim = 300
        vec_path = '../data/bookcorpus/pretrained.w2v.case/w2v_vec.npy'
        voc_path = '../data/bookcorpus/pretrained.w2v.case/w2v_voc.json'
        if os.path.isfile(voc_path):
            voc = json.load(open(voc_path))
            revoc = {v:k for k, v in voc.items()}
        voc_num_path = '../data/bookcorpus/pretrained.w2v.case/w2v_voc_num.json'
        if os.path.isfile(voc_num_path):
            word_num = {int(key):int(val) for key, val in json.load(open(voc_num_path)).items()}
    else:
        embedding_dim = 300
        voc_path = '../data/bookcorpus/scratch.w2v.200k.case/voc.json'
        if os.path.isfile(voc_path):
            voc = json.load(open(voc_path))
            revoc = {v:k for k, v in voc.items()}
        voc_num_path = '../data/bookcorpus/scratch.w2v.200k.case/voc_num.json'
        if os.path.isfile(voc_num_path):
            word_num = {int(key):int(val) for key, val in json.load(open(voc_num_path)).items()}
    
