import numpy as np 
from tqdm import *
import torch
from torch import nn
from torch import autograd
from torch.autograd import Variable
import torch.nn.functional as F
import numpy as np
import json
import random
import h5py
from torch.utils import data
import math
from nltk.tokenize import sent_tokenize
from nltk import word_tokenize
import lmdb
from torch.utils.data.dataloader import *

from config import Config
import copy


def _collate_fn(batch):
    
    '''过滤掉空字典或者长度太长的数据'''
    def filter_(x):
        if len(x) == 0:
            return False
        '''
        if x['intra_len'] > Config.max_sent_len:
            return False
        if max(x['ranking_length']) > Config.max_sent_len:
            return False
        '''
        return True
    
    batch = list(filter(filter_, batch))
    if len(batch) == 0:
        return {}
    
    max_seq_len = -1
    for i in range(len(batch)):
        max_in_one = batch[i]['intra_len']
        max_seq_len = max_in_one if max_in_one > max_seq_len else max_seq_len
        max_in_one = max(batch[i]['ranking_length'])
        max_seq_len = max_in_one if max_in_one > max_seq_len else max_seq_len
    max_seq_len = min(max_seq_len, Config.max_sent_len)
    
    
    for i in range(len(batch)):
        batch[i]['intra_input'] = batch[i]['intra_input'][:max_seq_len]
        batch[i]['order_input'] = batch[i]['order_input'][:max_seq_len]
        batch[i]['intra_len'] = min(max_seq_len, batch[i]['intra_len'])
        
        batch[i]['ranking_input'] = batch[i]['ranking_input'][:, :max_seq_len]
        for j in range(len(batch[i]['ranking_length'])):
            batch[i]['ranking_length'][j] = min(max_seq_len, batch[i]['ranking_length'][j])
        
    return default_collate(batch) # 用默认方式拼接过滤后的batch数据


class Utils:
    
    def __init__(self):
        
        self.init_ns()
    
    def init_ns(self):
        
        # 负采样，3/4次幂
        total_nums = 0
        # word_num: {word_id:word_count}
        for key, val in Config.word_num.items():
            total_nums += math.pow(val, 0.75)
        
        pw = [-1 for i in range(len(Config.word_num))]
        for key in sorted(Config.word_num.keys()):
            pw[key] = math.pow(Config.word_num[key], 0.75) / total_nums
        
        ns = np.array([0 for i in range(int(total_nums) + 1)])
        
        curr_word = 0
        curr_id = 0
        while curr_id < total_nums and curr_word < len(pw):
            len_ = round(pw[curr_word] * total_nums)
            ns[curr_id:curr_id+len_] = curr_word
            curr_id = curr_id + len_
            curr_word += 1
            if len_ < 1:
                print ('not good', curr_id, curr_word, len_)
        
        self.ns = ns
        self.ns_range = int(total_nums) + 1


    def t2i(self, text):
       
        text = text.split() if Config.case_sensitive else text.lower().split()
        ids = [Config.eos_id for i in range(Config.max_sent_len)]
        ids[0] = Config.eos_id
        
        whole = []
        
        for j, word in enumerate(text):
            whole.append(Config.voc.get(word, Config.unk_id))
            if j+1 < Config.max_sent_len:
                ids[j+1] = Config.voc.get(word, Config.unk_id)
            
        return np.array(ids), len(text)+1, np.array(whole)
    
    def i2t(self, ids):
       
        text = []
        for j, id_ in enumerate(ids):
            
            if Config.revoc[id_] == '<s>':
                continue
            if Config.revoc[id_] == '</s>':
                break
            
            text.append(Config.revoc[id_])
                
        return ' '.join(text)
    
    
    def t2i_sent(self, text):
        
        '''
        将句子id化
        '''
        text = text.rstrip('\n')
        word_ids, len_, whole_ids = self.t2i(text)
        return word_ids, len_, whole_ids
    
    def t2i_para(self, text):
        
        '''
        将段落id化，多次调用句子id化函数，若段落中句子数目太多，则随机若干连续句子
        '''
        sents = text.split('<sep>')
        num_sents = len(sents)
        while len(sents) < Config.std_num_ranking_seq:
            sents.append('')
        word_ids, lens = [], []
        start = random.randint(0, len(sents)-Config.std_num_ranking_seq)
        for sent in sents[start : start + Config.std_num_ranking_seq]:
            wid, wlen, whole_id = self.t2i_sent(sent)
            word_ids.append(wid)
            lens.append(wlen)
        
        return np.array(word_ids), np.array(lens), min(num_sents, Config.std_num_ranking_seq)
        
 
    def build_word_sample(self, ids, len_, whole_ids):
        '''
        ids: word2index，截断到最大模型输入最大长度，例如模型输入最大长度是30，原来的句子有60，那么ids最多也只有30长度
        len_: ids中原句子的长度，例如模型最大长度是30，而句子只有10，那么len_=10
        whole_ids: word2index，长度和原本句子一样，保留全部内容
        '''
        
        len_ = min(len_, Config.max_sent_len)
        '''限制正例只存在与ids中，即模型输入'''
        tmp = [ids[i] for i in range(len_) if ids[i] not in [Config.bos_id, Config.eos_id, Config.unk_id]]
        '''不限制正例只存在模型输入，而是出现在同一句话即可，即使被截断了'''
        #tmp = [whole_ids[i] for i in range(len(whole_ids)) if whole_ids[i] not in [Config.bos_id, Config.eos_id, Config.unk_id]]
        
        random.shuffle(tmp)
        pos_id = []
        while len(pos_id) < Config.std_word_pos_num:
            index_ = random.randint(0, len(tmp)-1)
            pos_id.append(tmp[index_])
        
        neg_id = []
        while(len(neg_id) < Config.std_word_neg_num):
            neg_id.append(self.ns[random.randint(0, self.ns_range-1)])
            if neg_id[-1] in ids:
                del neg_id[-1]
        
        return np.array(pos_id + neg_id), np.array([1] * len(pos_id) + [0] * len(neg_id))
    
    def build_order_sample(self, ids, len_):

        '''
        构造词序样本，正例是正常的句子，负例交换两个词的句子
        '''
        len_ = min(len_, Config.max_sent_len)
        
        shuffled_ids = [ids[i] for i in range(len(ids))]
        rand1, rand2 = 0, 0
        while rand2 == rand1:
            rand1 = random.randint(1, len_-1)
            rand2 = random.randint(1, len_-1)
        
        shuffled_ids[rand1], shuffled_ids[rand2] = shuffled_ids[rand2], shuffled_ids[rand1]
        label = 1 if shuffled_ids[rand1] == shuffled_ids[rand2] else 0
        
        return np.array(shuffled_ids), np.array([label])
        
    def build_ranking_sample(self, para, far_text):
        
        '''
        构造句子排序样本，（query，x1，x2），query为查询句子，x1是下一个句子，x2是其他位置的句子
        '''
        sents = para.rstrip('<sep>').split('<sep>')
        
        ids = [i for i in range(len(sents))]
        while True:
            random.shuffle(ids)
            if len(ids) < 3:
                break
            if abs(ids[0]-ids[1]) != abs(ids[0]-ids[2]):
                break
            
        query_input, query_len, _ = self.t2i_sent(sents[ids[0]])
        x1_input, x1_len, _ = self.t2i_sent(sents[ids[1]])
        if random.random() < Config.ns_para_ranking and len(ids) > 2:
            x2_input, x2_len, _ = self.t2i_sent(sents[ids[2]])
            label = 1 if abs(ids[0]-ids[1]) < abs(ids[0]-ids[2]) else -1
        else:
            x2_input, x2_len, _ = self.t2i_sent(far_text)
            label = 1
    
        return np.array([query_input, x1_input, x2_input]), np.array([query_len, x1_len, x2_len]), label
    

class Folder(data.Dataset):
    
    def __init__(self, filepath, is_test = False, num_samples=None, num_umbc=None):
        
        self.init_file(filepath, num_samples)
        self.is_test = is_test
        self.num_umbc = num_umbc # 由于数据来自bookcorpus和umbc，umbc在前，这个可以表示从第几条样本开始是bookcorpus
        self.util = Utils()
        
    def init_file(self, filepath, num_samples):
        
        self.env = lmdb.open(filepath)
        self.txn = self.env.begin()
        self.num_samples = self.txn.stat()['entries'] if not num_samples else min(self.txn.stat()['entries'], num_samples)
             
    
    def intra_sent_valid_index_bookcorpus(self, index):
        '''
        针对bookcorpus， 传入一个index，判断是否满足intra-sent任务
        合法index：合法的lmdb key， 并且句子的长度大于最小长度
        '''
        if not isinstance(index, bytes):
            index = index.encode()
        invalid = True
        text = self.txn.get(index)
        if not text:
            return False
        text = text.decode()
        if len(text.split()) > Config.min_sent_len:
            invalid = False
        return not invalid
        
    
    def intra_sent_valid_index_umbc(self, index):
        '''
        针对umbc，传入一个index，判断是否满足intra-sent任务
        合法index：合法的lmdb key，有一个句子的长度大于最小长度
        '''
        if not isinstance(index, bytes):
            index = index.encode()
        text = self.txn.get(index)
        if not text:
            return False
        text = text.decode()
        sents = text.rstrip('<sep>').split('<sep>')
        if len(sents) < 1:
            return False

        invalid = True
        for sent in sents:
            words = sent.split()
            if len(words) > Config.min_sent_len:
                invalid = False
                break

        return not invalid
        
    def intra_sent_valid_index(self, index):
        '''
        传入一个index(int)，返回一个可以正常查询的、满足intra-sent任务的index
        return：lmdb key，对应的样本长度大于最小输入长度
        '''
        
        while True:
            bindex = ('bookcorpus-' + str(index)).encode()
            is_valid = self.intra_sent_valid_index_bookcorpus(bindex)
            if is_valid:
                return bindex.decode()
            uindex = ('umbc-' + str(index)).encode()
            is_valid = self.intra_sent_valid_index_umbc(uindex)
            if is_valid:
                return uindex.decode()
            '''index 对应的样本不合法则随机一个新的index直到找到合法的样本'''
            index = random.randint(0, self.num_samples-1)
            
            
            
    def inter_sent_valid_index_bookcorpus(self, index):
        '''
        传入一个index(lmdb key)，针对bookcorpus，判断对应的样本是否满足inter-sent任务
        合法index：合法lmdb key， 并且后续句子数据要大于一个阈值，否则不能进行排序等任务
        '''
        if not isinstance(index, bytes):
            index = index.encode()
        invalid = True
        text = self.txn.get(index)
        if not text:
            return False
        text = text.decode()
        index_ = int(index.decode().split('-')[-1])
        if index_ + Config.text_window < self.num_samples and index_ - Config.text_window >= self.num_umbc:
            invalid = False
        return not invalid
    
    
    def inter_sent_valid_index_umbc(self, index):
        '''
        传入一个index(lmdb key)，针对umbc，判断对应的样本是否满足inter-sent任务
        合法index：合法lmdb key，并且样本包含的句子数大于一个阈值，否则不能进行排序等任务
        '''
        if not isinstance(index, bytes):
            index = index.encode()
        invalid = True
        text = self.txn.get(index)
        if not text:
            return False
        text = text.decode()
        sents = text.rstrip('<sep>').split('<sep>')
        if len(sents) > Config.min_num_ranking_seq:
            invalid = False
        return not invalid
        

    def inter_sent_valid_index(self, index):    
        '''
        传入一个index(int)，返回一个可以查询的、满足inter-sent任务的index
        句子数据要大于一个阈值，否则不能进行排序等任务
        '''
        while True:
            bindex = ('bookcorpus-' + str(index)).encode()
            is_valid = self.inter_sent_valid_index_bookcorpus(bindex)
            if is_valid:
                return bindex.decode()
            uindex = ('umbc-' + str(index)).encode()
            is_valid = self.inter_sent_valid_index_umbc(uindex)
            if is_valid:
                return uindex.decode()
            '''index 对应的样本不合法则随机一个新的index直到找到合法的样本'''
            index = random.randint(0, self.num_samples-1)
            

    def select_sent_umbc(self, key):
        '''
        输入一个umbc对应的key(lmdb key)，返回一个句子
        umbc key对应的样本的几句话构成的para，随机返回其中的一个（满足长度合法）
        '''
        para = self.txn.get(key).decode().rstrip('<sep>')
        sents = para.rstrip('<sep>').split('<sep>')
        while True:
            index = random.randint(0, len(sents)-1)
            if len(sents[index].split()) > Config.min_sent_len:
                break
        return sents[index]
    
    def select_para_bookcorpus(self, key):
        '''
        输入一个bookcorpus对应的可以（lmdb key）， 返回几个句子构成的para
        bookcorpus key对应的样本都是连续的单句，需要向下查询足够次数，构成para返回
        '''
        index = int(key.decode().split('-')[-1])
        para = []
        for i in range(-Config.text_window, Config.text_window+1):
            key = 'bookcorpus-' + str(index+i)
            tx = self.txn.get(str(key).encode()).decode().rstrip('<sep>')
            para.append(tx)
        return '<sep>'.join(para)
        
    
    def get_sent(self, key):
        
        '''
        根据key返回句子
        '''
        if not isinstance(key, bytes):
            key = key.encode()
        
        if 'umbc' in key.decode():
            return self.select_sent_umbc(key)
        elif 'bookcorpus' in key.decode():
            return self.txn.get(key).decode().rstrip('<sep>')
    
    def get_para(self, key):
        
        '''
        根据key返回段落
        '''
        if not isinstance(key, bytes):
            key = key.encode()
        if 'umbc' in key.decode():
            return self.txn.get(key).decode().rstrip('<sep>')
        elif 'bookcorpus' in key.decode():
            return self.select_para_bookcorpus(key)
    
    def random_far_index(self, index):
        
        '''
        为id为index的句子，采样一条足够远的句子，用于句子排序
        句子排序有时候可能难以区分比较相邻的句子，需要采用很不相邻的句子作为负例
        '''
        while True:
            far_index = random.randint(0, self.num_samples-1)
            far_index = self.intra_sent_valid_index(far_index)
            if 'umbc' in index and 'umbc' in far_index and far_index != index:
                break
            if 'bookcorpus' in index and 'bookcorpus' in far_index:
                far_ = int(far_index.split('-')[-1])
                index_ = int(index.split('-')[-1])
                if not far_ in range(index_-Config.text_window, index_+Config.text_window+1):
                    break
        return far_index
    
    def select_sent_from_para(self, para):
        
        '''
        从段落中随机一个句子
        '''
        sents = para.rstrip('<sep>').split('<sep>')
        if len(sents) == 1:
            return sents[0]
        while True:
            index = random.randint(0, len(sents)-1)
            if len(sents[index].split()) > Config.min_sent_len:
                break
        return sents[index].rstrip('<sep>')
    
        
    def __getitem__(self, index):

        try :
            intra_index = self.intra_sent_valid_index(index)
            inter_index = self.inter_sent_valid_index(index)
            
            intra_text = self.get_sent(intra_index)
            intra_input, seq_len, whole_ids = self.util.t2i_sent(intra_text)
            word_input, word_label = self.util.build_word_sample(intra_input, seq_len, whole_ids)
            order_input, order_label = self.util.build_order_sample(intra_input, seq_len)
            
            inter_text = self.get_para(inter_index)
            far_index = self.random_far_index(inter_index)
            far_text = self.select_sent_from_para(self.txn.get(str(far_index).encode()).decode())
            ranking_inputs, seq_lens, ranking_label = self.util.build_ranking_sample(inter_text, far_text)
            
            return {'intra_input': intra_input, 
                    'intra_len': seq_len,
                    'word_input':word_input,
                    'word_label':word_label,
                    'order_input':order_input,
                    'order_label':order_label,
                    'ranking_input':ranking_inputs, 
                    'ranking_label':ranking_label, 
                    'ranking_length':seq_lens}
        except:
            return {}

    
    def __len__(self):
        
        return self.num_samples
